// Correlator application
#include <qwidget.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include "tools.h"
#include "sndctrl.h"
#include "minenergy.h"
#include "debug.h"

#define DEBUG(x) 

#define CORRSIZE 8192
#define CORRSHIFT 72

#define MINDELAY 8192

#define FRAMEWIDTH 200

inline int energy(int channels, int frameSize, signed short ** data,
                  int rate, double phi, double theta)
{
#include "../lib/micpos.h"
#define MAXDELAY 30
   int k, a,b;
   long long sum=0;
   int delay[channels];
   int pairs=0;
   signed short *d0,*d1,*d2,*d3;
   // calculate the delays 
   double cp,sp,ct,st,dummy;
   cp=cos(phi);
   sp=sin(phi);
   ct=cos(theta);
   st=sin(theta);
   dummy=double(rate)/double(SPEEDOFSOUND);
   // turn the microphone virtually
   for (k=0; k<channels; k++) {
      delay[k]=int(rint((cp*(micpos[k].x*ct+micpos[k].z*st)-micpos[k].y*sp)
                        *dummy));
   }
   // process any possible microphone pair
   if (channels==4) {
      // unrolled version for 4 channels:
      d0=&data[0][MAXDELAY-delay[0]];
      d1=&data[1][MAXDELAY-delay[1]];
      d2=&data[2][MAXDELAY-delay[2]];
      d3=&data[3][MAXDELAY-delay[3]];
      for (k=MAXDELAY; k<frameSize-MAXDELAY; k++) {
         sum+=sqr((*d0)-(*d1));
         sum+=sqr((*d0)-(*d2));
         sum+=sqr((*d0++)-(*d3));
         sum+=sqr((*d1)-(*d2));
         sum+=sqr((*d1++)-(*d3));
         sum+=sqr((*d2++)-(*d3++));
      }
      pairs=6;
   } else {
      // normal version:
      for (a=0; a<channels-1; a++) {
         for (b=a+1; b<channels; b++) {
            for (k=MAXDELAY; k<frameSize-MAXDELAY; k++) {
               sum+=sqr(data[a][k-delay[a]]-data[b][k-delay[b]]);
            }
            pairs++;
         }
      }
   }
   // normalize and return it!
   return sum/pairs/(frameSize-2*MAXDELAY);
}


Minenergy::Minenergy(int horizontalAngle, int verticalAngle, bool verboseMode,
                     Sound * soundApp, QWidget * parent=NULL,
                     char * name=NULL)  : QFrame(parent, "Minenergy")
{
   int i;
   sampleRate=soundApp->getSampleRate();
   receiver=new SoundReceiver(soundApp,soundApp->getChannels());
   receiver->setFrameSize(CORRSIZE);
   receiver->setStepSize(CORRSIZE);
   verbose=verboseMode;
   hAngle=horizontalAngle;
   vAngle=verticalAngle;
   for (i=0; i<soundApp->getChannels(); i++) {
      receiver->setDelay(i,MINDELAY);
   }
   setCaption("Direction to source");
   setPalette( QPalette( QColor(255, 255, 255) ) );
   connect(receiver,SIGNAL(newDataReady(int,int, signed short **)),
           this,SLOT(newDataReady(int, int, signed short **)));
   if (verbose) {
       setCaption("Direction to source");
       setPalette( QPalette( QColor(255, 255, 255) ) );
       setFixedSize(200,200);
       setGeometry(25*soundApp->getChannels()+25,220,width(),height());
       show();
   }
   pos=0;

}


Minenergy::~Minenergy()
{
    delete receiver;
}


void Minenergy::newDataReady(int channels, int frameSize, signed short ** data)
{
   int phi,theta;
   int min,pmin,tmin;
   int result;

   DEBUG(fprintf(stderr,"Minenergy: data received!\n"));
   min=1000000000;
   pmin=-1000;
   tmin=0;
   for (theta=-vAngle; theta<=vAngle; theta+=4) {  
      for (phi=-hAngle;phi<=hAngle; phi+=4) {
         result=energy(channels, frameSize, data, sampleRate, double(phi)/180.0*M_PI, double(theta)/180.0*M_PI);
         if (result<min) {min=result; pmin=phi; tmin=theta;}
      }
   }
   if (pmin!=-1000) {
      p=double(pmin)/180.0*M_PI;
      t=double(tmin)/180.0*M_PI;
   }
   emit(sweepTo(t,p));
   repaint(TRUE);
}


void Minenergy::paintEvent(QPaintEvent *)
{
   if (verbose) {
      QPainter painter;
      painter.begin(this);
      if (vAngle==0) {
         painter.drawLine(width()/2,height()/2,
                          (width())/2+int(double(width())/2.0*cos(p)),
                          height()/2-int(double(height())/2.0*sin(p)));
      }
      else {
           painter.drawEllipse(width()/2*int(p/M_PI*180)/hAngle+width()/2-5,
                         width()/2*int(-t/M_PI*180)/vAngle+height()/2-5,10,10);
      }
      painter.end();
   }
}
