// Meter application

#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <qpainter.h>
#include <qpixmap.h>
#include "sndctrl.h"
#include "debug.h"
#include "oszi.h"

#define DEBUG(x) 

#define OSZISIZE 256

Oszi::Oszi(Sound * soundApp, QWidget * parent=NULL, char * name=NULL) 
    : QFrame(parent, name)
{
    // install a new handler for the sound data
    receiver=new SoundReceiver(soundApp,soundApp->getChannels());
    receiver->setFrameSize(OSZISIZE*4);
    receiver->setStepSize(OSZISIZE*8);
    connect(receiver,SIGNAL(newDataReady(int,int, signed short **)),
    	    this,SLOT(newDataReady(int, int, signed short **)));
    // the pixmap for the oszilloscope
    pm = new QPixmap(OSZISIZE,100);
    setFixedSize(OSZISIZE,100);
    setGeometry(25*soundApp->getChannels()+25,85,width(),height());
    setCaption("Oszilloscope");
    show();
    pm->fill();
    repaint();
}

Oszi::~Oszi()
{
    delete receiver;
}

void Oszi::newDataReady(int channels, int frameSize, signed short ** data)
{
    // data is available for oszilloscope, so process it!
    int i,k,trigger;
    char s[5];
    int oldv, newv;
    signed short * act;
    QPainter p;
    DEBUG(fprintf(stderr,"Oszi: data received!\n"));
    // clear the "screen"
    pm->fill();
    p.begin(pm);
    p.setPen(black);
    // paint zero-line into buffer
    p.drawLine(0,60,OSZISIZE-1,60);
    // search data for the first negative to positive zero-crossing
    // on channel 0 and use this as the trigger position
    trigger=0;
    for (i=1; (i<frameSize/2)&&(trigger==0); i++) {
	if ((data[0][i-1]<0) && (data[0][i+1]>0)) {
	    trigger=i;
	}
    }
    DEBUG(fprintf(stderr,"Oszi: Trigger is %d\n",trigger));
    for (i=0; i<channels; i++) {
	s[0]='A'+i; s[1]=0;
        // draw description
	p.setPen(QColor(360/channels*i,255,255,QColor::Hsv));
	p.drawText((OSZISIZE-35)/channels*i+10,15,s);
        act=&data[i][trigger];
        oldv=-(*act++)/820+60;
	for (k=0; k<frameSize/2-1; k++) {
            newv=-(*act++)/820+60;
	    p.drawLine(k,oldv,k+1,newv);
            oldv=newv;
	}
    }
    p.end();
    repaint(FALSE);
}

void Oszi::paintEvent(QPaintEvent *)
{
    // draw buffer to screen
    bitBlt(this,0,0,pm,0,0,pm->width(),pm->height(),CopyROP,TRUE);
}
