// Outputplugin application

#include "sndctrl.h"
#include "meter.h"
#include "outputplugin.h"
#include "debug.h"

#define FRAMEWIDTH 2048

#define MINDELAY 0

#define DEBUG(x) 

SweepControl::SweepControl(QWidget * parent=NULL, const char * name=NULL):
  QLabel(parent, name)
{
    phiBar=new QScrollBar (-180,180,1,10,0,QScrollBar::Horizontal,this,"phiBar");
    phiBar->setGeometry(5,25,200,15);
    connect(phiBar,SIGNAL(valueChanged(int)),this,SLOT(phiBarChanged(int)));
    thetaBar=new QScrollBar (-90,90,1,10,0,QScrollBar::Horizontal,this,"thetaBar");
    thetaBar->setGeometry(5,65,200,15);
    connect(thetaBar,SIGNAL(valueChanged(int)),this,SLOT(thetaBarChanged(int)));
    theta=0;
    phi=0;
    setFixedSize(210,90);
    setGeometry(400,220,width(),height());
    show(); 
    setCaption("Manual Sweep Control");
}


SweepControl::~SweepControl()
{
    delete thetaBar;
    delete phiBar;
}

void SweepControl::phiBarChanged(int p)
{
    phi=p;
    emit sweepTo(double(theta)/180.0*M_PI, double(phi)/180.0*M_PI);
    repaint(true);
}

void SweepControl::thetaBarChanged(int t)
{
   theta=t;
   emit sweepTo(double(theta)/180.0*M_PI, double(phi)/180.0*M_PI);
   repaint(true);
}

void SweepControl::paintEvent(QPaintEvent *)
{
   QPainter p;
   p.begin(this);
   char s[20];
   sprintf(s,"Phi:   %4d",phi);
   p.drawText(15,20,s);
   sprintf(s,"Theta: %4d",theta);
   p.drawText(15,60,s);
   p.end();
   
}

OutputPlugin::OutputPlugin(bool measuremode, int newRate, bool verbosemode, Sound * soundApp) 
    : QFrame(NULL, "Microphone Direction")
{
   verbose=verbosemode;
   active=true;
   oldphi=0;
   measure=measuremode;
   setCaption("Steered microphone");
   setPalette( QPalette( QColor(255, 255, 255) ) );
   receiver=new SoundReceiver(soundApp,soundApp->getChannels());
   receiver->setFrameSize(FRAMEWIDTH);
   receiver->setStepSize(FRAMEWIDTH);
   connect(receiver,SIGNAL(newDataReady(int,int, signed short **)),
           this,SLOT(newDataReady(int, int, signed short **)));
   newSampleRate=newRate;
   oldSampleRate=soundApp->getSampleRate();
   if (newSampleRate>oldSampleRate) {
      fprintf(stderr,"Outputplugin: upsampling not possible!\n");
      exit(1);
   }
   theta=0; phi=0;
   channels=soundApp->getChannels();
   if (verbose) {
      setCaption("Microphone direction");
      setGeometry(400,220,200,200);
      setFixedSize(width(),height());
      show();
   }
}


OutputPlugin::~OutputPlugin()
{
    delete receiver;
}


void OutputPlugin::newDataReady(int channels, int frameSize, signed short ** data)
{
    int i,k,c;
    static int count=0;
    static int sum=0, number=0;
    signed short dummy[frameSize+1];
    DEBUG(fprintf(stderr,"Outputplugin: data received!\n"));
    c=0;
    // Data is ready, so prepare it for output
    // process the whole frame
    for (i=0; i<frameSize; i++) {
        // sum all channels, the output is mono!
        for (k=0; k<channels; k++) {
            sum+=data[k][i];
            number++;
        }
        // if output samplerate is lower than working samplerate,
        // sum over multiple samples and output the sum when desired
        count+=newSampleRate;
        if (count>=oldSampleRate) {
            dummy[c]=(signed short)(sum/number);
            number=0;
            sum=0;
            count-=oldSampleRate;
            c++;
        }
    }
    write(1,(void*)dummy,c*2);
}


void OutputPlugin::sweepTo(double t, double p)
{
#include "../lib/micpos.h"
   // ignore signal if no speech present
   if (measure) {fprintf(stderr,"Phi:%4d\n",oldphi);}
   if (!active) {return;}
   oldphi=int(p*180.0/M_PI);
   int k;
   int min=1000;
   double cp,sp,ct,st;
   int delay[channels];
   DEBUG(fprintf(stderr,"Outputplugin: sweepTo-signal received!\n"));
   // calculate the delays for each channel
   cp=cos(p);
   sp=sin(p);
   ct=cos(t);
   st=sin(t);
   for (k=0; k<channels; k++) {
      delay[k]=int(rint((cp*(micpos[k].x*ct+micpos[k].z*st)-micpos[k].y*sp)+0.25));
      if (delay[k]<min) {min=delay[k];}
      DEBUG(fprintf(stderr,"%3d ",delay[k]));
   }
   DEBUG(fprintf(stderr,"\n"));
   // set the delays for each channel
   for (k=0; k<channels; k++) {
      receiver->setDelay(k,(delay[k]-min)*oldSampleRate/SPEEDOFSOUND+MINDELAY);
   }
   phi=p;
   if (verbose) {repaint(true);}
}


void OutputPlugin::speechActivityChanged(bool a)
{
    active=a;
}


void OutputPlugin::paintEvent(QPaintEvent *)
{
   if (verbose) {
      QPainter painter;
      painter.begin(this);
      painter.drawLine(width()/2,height()/2,
                       (width())/2+int(double(width())/2.0*cos(phi)),
                       height()/2-int(double(height())/2.0*sin(phi)));
      painter.end();
   }
}
