/*
************************************************************
* sndctrl.h                                                *
* extended functions to control soundcards                 *
* last change 2000-08-14                                   *
* Robin Gruber                                             *
************************************************************
*/

#ifndef SNDCTRL_H
#define SNDCTRL_H

#include <qobject.h>
#include <qscrollbar.h>
#include <qframe.h>
#include "../lib/soundlow.h"

// number of samples to be read at once
#define READSIZE 8192
// number of buffers used for the fifo
#define BUFFERSIZE 16
#define PRETRIGGER 10

#define MODE_NORMAL 0
#define MODE_SIMULATION 1
#define MODE_FILEIO 2

#define SIMLENGTH 61000

class Control : public QFrame
{
Q_OBJECT
 public:
    Control(QWidget * parent=NULL, const char * name=NULL);
    ~Control();
    QScrollBar * phiBar, *thetaBar;
 protected:
    void paintEvent(QPaintEvent *);
 private:
     int theta, phi; 
private slots:
     void phiBarChanged(int phi);
     void thetaBarChanged(int phi); 
}; 


class Sound  : public QObject
{
Q_OBJECT
 public:
    Sound(int channels, int rate, int simmode, char * inputfilename, QObject * parent=NULL, char * name=NULL);
    ~Sound();
    void startSampling();
    int getBufferPos();
    signed short getBuffer(int channel, int pos);
    int getChannels();
    int getSampleRate();
 public slots:
    void setSimTheta(int);
    void setSimPhi(int);
 signals:
    void newDataReady();
 private:
    signed short getSimValue(int,int);
    signed short wave[SIMLENGTH];
    int simPos;
    double simPhi,simTheta;
    Control * control;
    SoundCard * soundcard;
    static int count;
    int rate;
    bool samplingActive;
    int * fd;
    int cards;
    signed short * buffer;
    int bufferPos;
    int channels;
    void timerEvent(QTimerEvent *);
    int simmode;
    int * iofile;
};


class SoundReceiver : public QObject
{
Q_OBJECT
 public:
    SoundReceiver(Sound * soundApp, int channels);
    ~SoundReceiver();
    void setDelay(int delayNr, int delay);
    void setFrameSize(int frameSize);
    void setStepSize(int stepSize);
 public slots:
    void dataReady();
 signals:
    void newDataReady(int channels, int frameSize, signed short **data);
 private:
    int * delays;
    int channels;
    int frameSize;
    int stepSize;
    int readPos;
    short signed **data;
    Sound * soundApp;
};



#endif
