// *********************************************
// * Sound Library                             *
// * last change: 1998-10-09                   *
// * Robin Gruber                              *
// *********************************************
#include <termios.h>

#include "sound.h"


int openDSP(char * device, int mode, int width, int frequency, int channels)
{
    int id=-1;
    int sound_mode;
    int format;
    if (mode==RECORD) {id=open(device,O_RDONLY | O_NOCTTY,0);}
    if (mode==PLAYBACK)  {id=open(device,O_WRONLY | O_NOCTTY,0);}
    if (mode==DUPLEX)  {id=open(device,O_RDWR | O_NOCTTY,0);}
    if (id==-1) {return -2;} // Cannot open device!
    if (ioctl(id,SNDCTL_DSP_RESET,0)==-1) {return -5;} // IO-error
    sound_mode=0;
    if (width==8) {sound_mode=AFMT_U8;}
    if (width==16) {sound_mode=AFMT_S16_LE;}
    if (sound_mode==0) {return -6;} // no width given!
    format=sound_mode;
    if (ioctl(id,SNDCTL_DSP_SETFMT,&format)==-1) {return -5;} // IO-error
    if (format!=sound_mode) {return -7;} // Format not supported
    if (channels==STEREO) {sound_mode=1;} else {sound_mode=0;}
    format=sound_mode;
    if (ioctl(id,SNDCTL_DSP_STEREO,&format)==-1) {return -5;} // IO-error
    if (format!=sound_mode) {return -8;} // number of channels not supported
    format=frequency;
    if (ioctl(id,SNDCTL_DSP_SPEED,&format)==-1) {return -5;} //IO-error
    if (format!=frequency) {return -9;} // sample-frequency not supported
    return id;
}

int closeDSP(int id)
{
    if (close(id)==-1) {return -5;} // IO-error;
    return 0;
}

int record(int id, char * data, int length)
{
    if (read(id, data, length)==-1) {return -5;} // IO-error
    return 0;
}

int playback(int id, char * data, int length)
{
    if (write(id, data, length)==-1) {return -5;} // IO-error
    return 0;
}

