/*
 *  quadrec.c - record quadrophonic
 *
 *  quadrec (quadrophonic sound recorder) by Robin Gruber <archi@robinsnest.de>
 */


#include <stdio.h>
#include <stdlib.h>

#include "../lib/soundlow.h"


#define DEFAULT_SPEED 		48000
#define CALIBRUNS               1000
#define BUFFERSIZE              32768 
#define CHANNELS                4


inline int corrmax(signed short * a, signed short * b, int size, int shift)
{
    int i,k,ashift,bshift;
    long long r;
    int max=0;
    int maxi=0xffffffff;
    for (i=0; i<shift; i++) {
        r=0;
        ashift=i/2;
        bshift=shift/2-(i+1)/2;
        for (k=0; k<(size-shift/2); k++) {
            r+=a[ashift+k]*b[bshift+k];
        }
        if (r>max) {max=r; maxi=i;}
    }
    return maxi-shift/2;
}


int main(int argc, char *argv[])
{
    int k,l;
    signed short * data;
    signed short buffer[CHANNELS][BUFFERSIZE];
    // initialize soundcard
    SoundCard soundcard((CHANNELS+1)/2,DEFAULT_SPEED,BUFFERSIZE);
    data=(signed short*)malloc((CHANNELS+1)/2*2*sizeof(short));
    printf("Recording...\n");

    // start recording
    soundcard.go();
    while (true) {
        soundcard.read(data);
        for (k=0; k<CHANNELS; k++) {
            for (l=0; l<BUFFERSIZE; l++) {
                buffer[k][l]=data[(k+1)/2*BUFFERSIZE*2+l*2+k%2];
            }
        }
        printf("Channel 1 <-> Channel 3: %3d, ", corrmax(buffer[0],buffer[2],2000,20));
        printf("Channel 1 <-> Channel 2: %3d, ", corrmax(buffer[0],buffer[1],2000,20));
        printf("Channel 3 <-> Channel 4: %3d\n", corrmax(buffer[2],buffer[3],2000,20));
    }
    return 0;
}
